<?php

namespace Laraautos\Car\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Laraautos\Car\Forms\Car as CarForm;
use Laraautos\Car\Http\Requests\CarRequest;
use Laraautos\Car\Interfaces\CarRepositoryInterface;
use Laraautos\Car\Repositories\Eloquent\Filters\CarResourceFilter;
use Laraautos\Car\Repositories\Eloquent\Presenters\CarListPresenter;

/**
 * Resource controller class for car.
 */
class CarResourceController extends BaseController
{

    /**
     * Initialize car resource controller.
     *
     *
     * @return null
     */
    public function __construct(CarRepositoryInterface $car)
    {
        parent::__construct();
        $this->form = CarForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('laraautos.car.modules'), 'car', guard_url('car'));
        $this->repository = $car;
    }

    /**
     * Display a list of car.
     *
     * @return Response
     */
    public function index(CarRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CarResourceFilter::class)
            ->setPresenter(CarListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('car::car.names'))
            ->view('car::car.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display car.
     *
     * @param Request $request
     * @param Model   $car
     *
     * @return Response
     */
    public function show(CarRequest $request, CarRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('car::car.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('car::car.show')
            ->output();
    }

    /**
     * Show the form for creating a new car.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CarRequest $request, CarRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('car::car.name'))
            ->view('car::car.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new car.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CarRequest $request, CarRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('car::car.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('car/car/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/car/car'))
                ->redirect();
        }

    }

    /**
     * Show car for editing.
     *
     * @param Request $request
     * @param Model   $car
     *
     * @return Response
     */
    public function edit(CarRequest $request, CarRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('car::car.name'))
            ->view('car::car.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the car.
     *
     * @param Request $request
     * @param Model   $car
     *
     * @return Response
     */
    public function update(CarRequest $request, CarRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('car::car.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('car/car/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('car/car/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the car.
     *
     * @param Model   $car
     *
     * @return Response
     */
    public function destroy(CarRequest $request, CarRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('car::car.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('car/car/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('car/car/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
